Improvments by Matt for the --progress option, including updating
[rsync/rsync.git] / flist.c
diff --git a/flist.c b/flist.c
index 6f1033a..979090e 100644 (file)
--- a/flist.c
+++ b/flist.c
@@ -90,15 +90,15 @@ static int show_filelist_p(void)
 
 static void start_filelist_progress(char *kind)
 {
-       rprintf(FINFO, "%s ... ", kind);
+       rprintf(FCLIENT, "%s ... ", kind);
        if (verbose > 1 || do_progress)
-               rprintf(FINFO, "\n");
+               rprintf(FCLIENT, "\n");
        rflush(FINFO);
 }
 
 static void emit_filelist_progress(int count)
 {
-       rprintf(FINFO, " %d files...\r", count);
+       rprintf(FCLIENT, " %d files...\r", count);
 }
 
 static void maybe_emit_filelist_progress(int count)
@@ -149,61 +149,53 @@ static void list_file_entry(struct file_struct *f)
        }
 }
 
-/**
- * Stat either a symlink or its referent, depending on the settings of
- * copy_links, copy_unsafe_links, etc.
- *
- * @retval -1 on error
- *
- * @retval 0 for success
+/* Stat either a symlink or its referent, depending on the settings of
+ * copy_links, copy_unsafe_links, etc.  Returns -1 on error, 0 on success.
  *
- * @post If @p path is a symlink, then @p linkbuf (of size @c
- * MAXPATHLEN) contains the symlink target.
+ * If path is the name of a symlink, then the linkbuf buffer (which must hold
+ * MAXPATHLEN chars) will be set to the symlink's target string.
  *
- * @post @p buffer contains information about the link or the
- * referrent as appropriate, if they exist.
- **/
-static int readlink_stat(const char *path, STRUCT_STAT *buffer, char *linkbuf)
+ * The stat structure pointed to by stp will contain information about the
+ * link or the referent as appropriate, if they exist. */
+static int readlink_stat(const char *path, STRUCT_STAT *stp, char *linkbuf)
 {
 #ifdef SUPPORT_LINKS
-       if (copy_links)
-               return do_stat(path, buffer);
-       if (link_stat(path, buffer, copy_dirlinks) < 0)
+       if (link_stat(path, stp, copy_dirlinks) < 0)
                return -1;
-       if (S_ISLNK(buffer->st_mode)) {
-               int l = readlink((char *)path, linkbuf, MAXPATHLEN - 1);
-               if (l == -1)
+       if (S_ISLNK(stp->st_mode)) {
+               int llen = readlink(path, linkbuf, MAXPATHLEN - 1);
+               if (llen < 0)
                        return -1;
-               linkbuf[l] = 0;
+               linkbuf[llen] = '\0';
                if (copy_unsafe_links && unsafe_symlink(linkbuf, path)) {
                        if (verbose > 1) {
                                rprintf(FINFO,"copying unsafe symlink \"%s\" -> \"%s\"\n",
                                        path, linkbuf);
                        }
-                       return do_stat(path, buffer);
+                       return safe_stat(path, stp);
                }
        }
        return 0;
 #else
-       return do_stat(path, buffer);
+       return do_stat(path, stp);
 #endif
 }
 
-int link_stat(const char *path, STRUCT_STAT *buffer, int follow_dirlinks)
+int link_stat(const char *path, STRUCT_STAT *stp, int follow_dirlinks)
 {
 #ifdef SUPPORT_LINKS
        if (copy_links)
-               return do_stat(path, buffer);
-       if (do_lstat(path, buffer) < 0)
+               return safe_stat(path, stp);
+       if (do_lstat(path, stp) < 0)
                return -1;
-       if (follow_dirlinks && S_ISLNK(buffer->st_mode)) {
+       if (follow_dirlinks && S_ISLNK(stp->st_mode)) {
                STRUCT_STAT st;
-               if (do_stat(path, &st) == 0 && S_ISDIR(st.st_mode))
-                       *buffer = st;
+               if (safe_stat(path, &st) == 0 && S_ISDIR(st.st_mode))
+                       *stp = st;
        }
        return 0;
 #else
-       return do_stat(path, buffer);
+       return do_stat(path, stp);
 #endif
 }
 
@@ -290,7 +282,7 @@ void flist_expand(struct file_list *flist)
                                flist->malloced);
 
        if (verbose >= 2 && flist->malloced != FLIST_START) {
-               rprintf(FINFO, "[%s] expand file_list to %.0f bytes, did%s move\n",
+               rprintf(FCLIENT, "[%s] expand file_list to %.0f bytes, did%s move\n",
                    who_am_i(),
                    (double)sizeof flist->files[0] * flist->malloced,
                    (new_ptr == flist->files) ? " not" : "");
@@ -322,7 +314,7 @@ static void send_file_entry(struct file_struct *file, int f)
        if (!file) {
                write_byte(f, 0);
                modtime = 0, mode = 0;
-               dev = 0, rdev = makedev(0, 0);
+               dev = 0, rdev = MAKEDEV(0, 0);
                rdev_major = 0;
                uid = 0, gid = 0;
                *lastname = '\0';
@@ -354,7 +346,7 @@ static void send_file_entry(struct file_struct *file, int f)
                                flags |= XMIT_RDEV_MINOR_IS_SMALL;
                }
        } else if (protocol_version < 28)
-               rdev = makedev(0, 0);
+               rdev = MAKEDEV(0, 0);
        if (file->uid == uid)
                flags |= XMIT_SAME_UID;
        else
@@ -505,7 +497,7 @@ static struct file_struct *receive_file_entry(struct file_list *flist,
 
        if (!flist) {
                modtime = 0, mode = 0;
-               dev = 0, rdev = makedev(0, 0);
+               dev = 0, rdev = MAKEDEV(0, 0);
                rdev_major = 0;
                uid = 0, gid = 0;
                *lastname = '\0';
@@ -538,7 +530,7 @@ static struct file_struct *receive_file_entry(struct file_list *flist,
        clean_fname(thisname, 0);
 
        if (sanitize_paths)
-               sanitize_path(thisname, thisname, "", 0);
+               sanitize_path(thisname, thisname, "", 0, NULL);
 
        if ((basename = strrchr(thisname, '/')) != NULL) {
                dirname_len = ++basename - thisname; /* counts future '\0' */
@@ -582,10 +574,10 @@ static struct file_struct *receive_file_entry(struct file_list *flist,
                                rdev_minor = read_byte(f);
                        else
                                rdev_minor = read_int(f);
-                       rdev = makedev(rdev_major, rdev_minor);
+                       rdev = MAKEDEV(rdev_major, rdev_minor);
                }
        } else if (protocol_version < 28)
-               rdev = makedev(0, 0);
+               rdev = MAKEDEV(0, 0);
 
 #ifdef SUPPORT_LINKS
        if (preserve_links && S_ISLNK(mode)) {
@@ -663,8 +655,8 @@ static struct file_struct *receive_file_entry(struct file_list *flist,
        if (linkname_len) {
                file->u.link = bp;
                read_sbuf(f, bp, linkname_len - 1);
-               if (sanitize_paths)
-                       sanitize_path(bp, bp, "", lastdir_depth);
+               if (lp_munge_symlinks(module_id))
+                       sanitize_path(bp, bp, "", lastdir_depth, NULL);
                bp += linkname_len;
        }
 #endif
@@ -745,7 +737,7 @@ struct file_struct *make_file(char *fname, struct file_list *flist,
        }
        clean_fname(thisname, 0);
        if (sanitize_paths)
-               sanitize_path(thisname, thisname, "", 0);
+               sanitize_path(thisname, thisname, "", 0, NULL);
 
        memset(sum, 0, SUM_LENGTH);
 
@@ -849,7 +841,7 @@ struct file_struct *make_file(char *fname, struct file_list *flist,
                ? MD4_SUM_LENGTH : 0;
 
        alloc_len = file_struct_len + dirname_len + basename_len
-           + linkname_len + sum_len;
+                 + linkname_len + sum_len;
        if (flist)
                bp = pool_alloc(flist->file_pool, alloc_len, "make_file");
        else {
@@ -930,7 +922,7 @@ struct file_struct *make_file(char *fname, struct file_list *flist,
                int save_mode = file->mode;
                file->mode = S_IFDIR; /* Find a directory with our name. */
                if (flist_find(the_file_list, file) >= 0
-                   && do_stat(thisname, &st2) == 0 && S_ISDIR(st2.st_mode)) {
+                   && safe_stat(thisname, &st2) == 0 && S_ISDIR(st2.st_mode)) {
                        file->modtime = st2.st_mtime;
                        file->length = st2.st_size;
                        file->mode = st2.st_mode;
@@ -1068,6 +1060,7 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
        int64 start_write;
        int use_ff_fd = 0;
 
+       rprintf(FLOG, "building file list\n");
        if (show_filelist_p())
                start_filelist_progress("building file list");
 
@@ -1078,6 +1071,8 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 
        io_start_buffering_out();
        if (filesfrom_fd >= 0) {
+               if (sanitize_paths)
+                       die_on_unsafe_path(argv[0], 0);
                if (argv[0] && !push_dir(argv[0])) {
                        rsyserr(FERROR, errno, "push_dir %s failed",
                                full_fname(argv[0]));
@@ -1094,13 +1089,13 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
                if (use_ff_fd) {
                        if (read_filesfrom_line(filesfrom_fd, fbuf) == 0)
                                break;
-                       sanitize_path(fbuf, fbuf, "", 0);
+                       sanitize_path(fbuf, fbuf, "", 0, NULL);
                } else {
                        if (argc-- == 0)
                                break;
                        strlcpy(fbuf, *argv++, MAXPATHLEN);
                        if (sanitize_paths)
-                               sanitize_path(fbuf, fbuf, "", 0);
+                               sanitize_path(fbuf, fbuf, "", 0, NULL);
                }
 
                len = strlen(fbuf);
@@ -1131,6 +1126,8 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
                                   && (len == 1 || fbuf[len-2] == '/');
                }
 
+               if (sanitize_paths)
+                       die_on_unsafe_path(fbuf, 1);
                if (link_stat(fbuf, &st, copy_dirlinks) != 0) {
                        io_error |= IOERR_GENERAL;
                        rsyserr(FERROR, errno, "link_stat %s failed",
@@ -1188,6 +1185,8 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
                                } else
                                        break;
                        }
+                       if (len == 1 && fn[0] == '/')
+                               fn[len++] = '.';
                        fn[len] = '\0';
                        /* Reject a ".." dir in the active part of the path. */
                        for (p = fn; (p = strstr(p, "..")) != NULL; p += 2) {
@@ -1334,6 +1333,7 @@ struct file_list *recv_file_list(int f)
        unsigned short flags;
        int64 start_read;
 
+       rprintf(FLOG, "receiving file list\n");
        if (show_filelist_p())
                start_filelist_progress("receiving file list");